/* 	$Id: id3tag.c,v 1.5 2000/01/08 20:00:26 ben Exp $	 */

#include  <stdio.h>
#include  <stdlib.h>
#include  <string.h>
#include  <ctype.h>
#include  <unistd.h>
#include  <sys/stat.h>
#include  <fcntl.h>


#include  "id3tag.h"

 /* Field  Length
 *  Tag      3      0-2      -128
 *  Title   30      3-32     -125
 *  Artist  30      33-62     -95
 *  Album   30      63-92     -65
 *  Year     4      93-96     -35
 *  Comment 30      97-126    -31
 *  Genre    1      127        -1
 */

struct id3tag id3tag_get( char *filename )
{
  struct id3tag         tag; 
  FILE                  *file;

  
  tag.tag[0]='\0';
  tag.tag[1]='\0';
  tag.tag[2]='\0';
  tag.tag[3]='\0';
  
  if ( (file=fopen(filename, "r")) == 0)
    {
      printf("\n");
      printf("id3tag.c, id3tag_get: ERROR while opening %s!\n", filename);
      return(tag);
    }

  /* Get the ID3 TAG */
  fseek(file, -128, SEEK_END);

  fread( &(tag.tag),      1,  3, file);

  if( strncmp(tag.tag, "TAG", 3) !=0 )
    {
      printf("WARNING: %s has no ID3 TAG!\n", filename);
      tag.tag[0]='\0';
      tag.tag[1]='\0';
      tag.tag[2]='\0';
      tag.tag[3]='\0';
      return(tag);
    }

  fread( &(tag.title),    1, 30, file);  tag.title[30]='\0';
  fread( &(tag.artist),   1, 30, file);  tag.artist[30]='\0';
  fread( &(tag.album),    1, 30, file);  tag.album[30]='\0';
  fread( &(tag.year),     1,  4, file);  tag.year[4]='\0';
  fread( &(tag.comment),  1, 30, file);  tag.comment[30]='\0';
  fread( &(tag.genre),    1,  1, file);

  /* Cut off spaces at the end */
  id3tag_cut( tag.title );
  id3tag_cut( tag.artist );
  id3tag_cut( tag.album );
  id3tag_cut( tag.year );
  id3tag_cut( tag.comment );
  
  fclose(file);
  return(tag);
}


int id3tag_show( struct id3tag tag )
{
  printf("Title:    %s\n", tag.title); 
  printf("Artist:   %s\n", tag.artist); 
  printf("Album:    %s\n", tag.album);
  printf("Year:     %s \t\tGenre: %s\n", tag.year, (tag.genre < 0 ) ? "?" : id3tag_genre_names[tag.genre] );
  printf("Comment:  %s\n", tag.comment);
  return(1);
}


int id3tag_new( char *filename )
{
  FILE *file;
  struct id3tag tag;

  if ( (file=fopen(filename, "w+")) == 0)
    {
      printf("\n");
      printf("id3tag.c, id3tag_new: ERROR while opening %s!\n", filename);
      return(-1);
    }


  strcpy( tag.tag, "TAG" );
  strncpy( tag.title, filename, 29 );
  tag.artist[0]=' ';  tag.artist[1]='\0';
  tag.album[0]=' ';   tag.album[1]='\0';
  tag.comment[0]=' '; tag.comment[1]='\0';
  tag.year[0]=' ';    tag.year[1]='\0';
  tag.genre=EOF;
  
  /* Write the ID3 TAG */
  fseek( file, 0, SEEK_END );
  
  fwrite( (tag.tag)     ,   1,   3, file);
  fwrite( (tag.title)   ,   1,  30, file);
  fwrite( (tag.artist)  ,   1,  30, file);
  fwrite( (tag.album)   ,   1,  30, file);
  fwrite( (tag.comment) ,   1,  30, file);
  fwrite( (tag.year)    ,   1,   4, file);

  fwrite( &(tag.genre)  ,   1,   1, file);

  fclose(file);
  
  printf("Writing a new ID3 TAG to %s...\n", filename);

  
  return(1);
}


int id3tag_set( char *filename, struct id3tag tag )
{
  FILE *file;

  
  if ( (file=fopen(filename, "r+")) == 0)
    {
      printf("\n");
      printf("id3tag.c, id3tag_set: ERROR while opening %s!\n", filename);
      return(-1);
    }
  
  /* Write the ID3 TAG */
  fseek(file, -128, SEEK_END);
  fwrite( (tag.tag),      1,  3, file);

  fseek( file, -1, SEEK_END);
  if( tag.genre < GENRE_MAX )
    fwrite( &(tag.genre),     1,  1, file);

  fseek( file, -65, SEEK_END);
  if( tag.album[0] != '\0' )
    {
      id3tag_fill( tag.album, 30 );
      fwrite( (tag.album),    1, 30, file);
    }
  
  fseek( file, -95, SEEK_END);
  if( tag.artist[0] != '\0' )
    {
      id3tag_fill( tag.artist, 30 );
      fwrite( (tag.artist),   1, 30, file);
    }
  
  fseek( file, -125, SEEK_END);
  if( tag.title[0] != '\0' )
    {
      id3tag_fill( tag.title, 30 );
      fwrite( (tag.title),    1, 30, file);
    }

  fseek( file, -31, SEEK_END);
  if( tag.comment[0] != '\0' )
    {
      id3tag_fill( tag.comment, 30 );
      fwrite( (tag.comment),  1, 30, file);
    }
  
  fseek( file, -35, SEEK_END);
  if( tag.year[0] != '\0' )
    {
      id3tag_fill( tag.year, 4 );
      fwrite( (tag.year),     1,  4, file);
    }
  
  fclose(file);
  return(1);
}


int id3tag_rename( char *filename, char *format )
{
  struct id3tag tag;
  struct stat stbuf;
  char target_filename[80]="";
  char buffer[10]="";
  int i;
  int i2;

  
  tag=id3tag_get( filename );

  /* If the file has no valid tag */

  if(strncmp(tag.tag, "TAG", 3) != 0 )
    {
      printf("id3tag.c, id3tag_rename: Skipped \"%s\". No valid tag!\n", filename);
      return(0);
    }
  
  /* The spaces at the end of the strings must be removed */
  id3tag_cut( tag.title );  
  id3tag_cut( tag.artist );
  id3tag_cut( tag.album );
  id3tag_cut( tag.comment );
   
  /* See what the user wants the format to be */
  for(i=0, i2=0; i<strlen(format); i++)
    {
      if( format[i] == '%' && format[i+1] != '%' && format[i-1] != '%')
	{
	  switch (format[i+1] )
	    {
	    case 't':
	      strcat( target_filename, tag.title);
	      i2=i2+strlen(tag.title);
	      i++;
	      break;
	      
	    case 'a':
	      strcat( target_filename, tag.artist);
	      i2=i2+strlen(tag.artist);
	      i++;	      
	      break;

	    case 'b':
	      strcat( target_filename, tag.album);
	      i2=i2+strlen(tag.album);
	      i++;	      
	      break;

	    case 'c':
	      strcat( target_filename, tag.comment);
	      i2=i2+strlen(tag.comment);
	      i++;	      
	      break;
		
	    case 'y':
	      strcat( target_filename, tag.year);
	      i2=i2+strlen(tag.year);
	      i++;	      
	      break;

	    case 'g':
	      strcat( target_filename, id3tag_genre_names[tag.genre]);
	      i2=i2+strlen(id3tag_genre_names[tag.genre]);	      
	      i++;	      
	      break;
		
	    default:
	      printf("Format Argument not found: %c !\n", format[i+1] );
	      return(-1);
	      break;
		
	    } /* End switch */
	} /* End if */
      else
	{
	  target_filename[i2]=format[i];
	  i2++;
	}
    } /* End for */
		
/*   id3tag_show( tag ); */

  /* Use .mp3 as the file extension */
  strcat( target_filename, ".mp3" );
  
/*   printf("target_filename: %s\n", target_filename); */

  for( i=1; ;i++)
    {
      /* If the file existst */
      if (stat( target_filename, &stbuf ) == 0)
	{
	  printf("id3tag.c, id3tag_rename: ERROR: File \"%s\" exists already!\n", target_filename);
	  target_filename[strlen(target_filename)-4]='\0';
	  sprintf( buffer, " #%i", i);
	  strcat( target_filename, buffer );
	  strcat( target_filename, ".mp3" );
	}
      else
	break;
    }
  
  if( (rename( filename, target_filename )) )  
    {  
      printf("id3tag.c, id3tag_rename: ERROR while renaming %s to %s!\n", filename, target_filename); 
      return(-1);  
    } 
  printf("File \"%s\" moved to \"%s\".\n", filename, target_filename);
  
  return(1);
}


int id3tag_sort( char *filename, char *rootdir, char *format_level1, char *format_level2 )
{
  struct id3tag tag;
  char *dir_level1;
  char *dir_level2;
  char target_filename[80];
  char dir_cur[80];

  
  getcwd( dir_cur, sizeof(dir_cur) );
  printf("Current dir: %s\n", dir_cur );
  tag = id3tag_get( filename );
  /* Check if the file has a valid ID3 TAG */
  if( strncmp(tag.tag,"TAG",3) != 0 )
    {
      printf("id3tag.c, id3tag_sort: %s not sorted!\n", filename );
      return(0);
    }
  
  /* Level 1 */
  if( strcmp( format_level1, "Artist") == 0 )
    dir_level1 = tag.artist;
  else if( strcmp( format_level1, "Album") == 0 )
    dir_level1 = tag.album;
  else if( strcmp( format_level1, "Year") == 0 )
    dir_level1 = tag.year;
  else if( strcmp( format_level1, "Genre") == 0 )
    {
      if( tag.genre == EOF )
	dir_level1 = "Unknown";
      else
	dir_level1 = id3tag_genre_names[tag.genre];
    } 
  else if( strcmp( format_level1, "Comment") == 0 )
    dir_level1 = tag.comment;
  else if( strcmp( format_level1, "Title") == 0 )
    dir_level1 = tag.title;
      
  /* Go to the root directory */
  chdir(rootdir);
  if( dir_level1[0] == '\0' )
    dir_level1 = "Unknown";
  printf("New directory: %s\n", dir_level1);
  mkdir( dir_level1, S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH );
	  
  /* Level 2 */
  if( strcmp( format_level2, "Album") == 0 )
    dir_level2 = tag.album;
  else if( strcmp( format_level2, "Artist") == 0 )
    dir_level2 = tag.artist;
  else if( strcmp( format_level2, "Year") == 0 )
    dir_level2 = tag.year;
  else if( strcmp( format_level2, "Genre") == 0 )
    {
      if( tag.genre == EOF )
	dir_level2 = "Unknown";
      else
	dir_level2 = id3tag_genre_names[tag.genre];
    } 
  else if( strcmp( format_level2, "Comment") == 0 )
    dir_level2 = tag.comment;
  else if( strcmp( format_level2, "Title") == 0 )
    dir_level2 = tag.title;
  
  if( dir_level2[0] == '\0' )
    dir_level2 = "Unknown";
  printf("New directory: %s\n", dir_level2);
  /* Go into the previously created directory */
  chdir( dir_level1 );
  mkdir( dir_level2, S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH );
  /* Move the file into the new (?) directory */
  /* BETA !!! */

  sprintf( target_filename, "mv \"%s/%s\" \"%s%s/%s/%s\"", dir_cur, filename, rootdir, dir_level1, dir_level2, filename ); 

  printf("target_filename: %s\n", target_filename); 

  system( target_filename ); 
  
/*   if( (rename( filename, target_filename )) )   */
/*     {   */
/*       printf("id3tag.c, id3tag_sort: ERROR while renaming %s to %s!\n", filename, target_filename);  */
/*       return(-1);   */
/*     }    */
  return(1);
}


int id3tag_cmp( struct id3tag tag, struct id3tag tag2 )
{
  int result=0;
  int no=0;     /*Will be set to 1 if a strcmp returned not 0 */

  
  if( tag.title[0] != '\0' && no == 0 )
    result = strcmp( tag.title, tag2.title);
  if( result != 0 )
    no = 1;
   
  if( tag.artist[0] != '\0' && no == 0 )
    result = strcmp( tag.artist, tag2. artist);
  if( result != 0 )
    no = 1;
  
  if( tag.album[0] != '\0' && no == 0 )
    result = strcmp( tag.album, tag2.album );
  if( result != 0 )
    no = 1;
  
  if( tag.comment[0] != '\0' && no == 0 )
    result = strcmp( tag.comment, tag2.comment );
  if( result != 0 )
    no = 1;
  
  if( tag.year[0] != '\0' && no == 0 )
    result = strcmp( tag.year, tag2.year );
  if( result != 0 )
    no = 1;
  
   if( tag.genre > 0 && no == 0 ) 
     { 
      if( tag.genre == tag2.genre)
	  result = 0;
      else
	result = 1;
     }
  if( result != 0 )
    no = 1;
	  
  if( no == 1 )
    result = 1;
      
  return(result);
}


void id3tag_cut( char *string )
{
  int i;

  
  for( i=strlen(string); !(isalpha(string[i])) && !(isalnum(string[i])) ; i-- )
    if( i < 0 )
      break;
  string[++i]='\0';
}


void id3tag_fill( char *string, int end )
{
  int i;

  
  for( i=strlen(string); i<end ; i++ )
    {
      string[i]=' ';
    }
  string[i]='\0';
}
